package com.foreverwin.mesnac.dispatch.service.impl;

import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.dto.SfcDispatchDto;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.dispatch.dto.IssueItemDto;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.dispatch.model.CallItem;
import com.foreverwin.mesnac.dispatch.mapper.CallItemMapper;
import com.foreverwin.mesnac.dispatch.service.CallItemService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author Leon.L
 * @since 2021-06-30
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class CallItemServiceImpl extends ServiceImpl<CallItemMapper, CallItem> implements CallItemService {


    @Autowired
    private CallItemMapper callItemMapper;

    @Override
    public void saveCallItem(String site, String user, List<SfcDispatchDto> sfcDispatchList) {
        if (sfcDispatchList == null || sfcDispatchList.size() <= 0) {
            return;
        }
        LocalDateTime nowDate = LocalDateTime.now();

        //查询叫料组件
        List<CallItem> callItemList = callItemMapper.selectDispatchCallItem(sfcDispatchList);
        if (callItemList == null || callItemList.size() <= 0) {
            throw BusinessException.build("发布的派工单没有物料清单组件");
        }
        List<CallItem> list = new ArrayList<>();

        //删除原来的数据
        callItemMapper.deleteDispatchCallItem(sfcDispatchList);

        //工序叫料数据
        for (SfcDispatchDto sfcDispatchDto : sfcDispatchList) {
            String prepositionStepId = sfcDispatchDto.getPrepositionStepId();
            if (StringUtil.isBlank(prepositionStepId)) {
                continue;
            }

            String callItemNo = sfcDispatchDto.getSfc() + sfcDispatchDto.getStepId() + prepositionStepId;
            CallItem callItem = new CallItem();
            callItem.setCallItemNo(callItemNo);
            callItem.setHandle(HandleEnum.CALL_ITEM.getHandle(site, callItemNo));
            callItem.setSite(site);
            callItem.setShopOrder(sfcDispatchDto.getShopOrder());
            callItem.setSfc(sfcDispatchDto.getSfc());
            callItem.setSfcDispatchBo(sfcDispatchDto.getHandle());
            callItem.setOperation(sfcDispatchDto.getOperation());
            callItem.setStepId(sfcDispatchDto.getStepId());
            callItem.setResrce(sfcDispatchDto.getResrce());
            callItem.setCallType(Constants.CALL_TYPE_OPERATION);
            callItem.setComponentBo(sfcDispatchDto.getItemBo());
            callItem.setRequiredQty(sfcDispatchDto.getDispatchQty());
            callItem.setRequiredDateTime(sfcDispatchDto.getPlannedStartDate());
            callItem.setStatus(Constants.STATUS_NEW);
            callItem.setCallStatus(Constants.STATUS_NEW);
            callItem.setIssueStatus(Constants.STATUS_NEW);
            callItem.setCreateUser(user);
            callItem.setCreatedDateTime(nowDate);
            callItem.setUpdateUser(user);
            callItem.setUpdatedDateTime(nowDate);
            list.add(callItem);
        }


        //组件叫料数据
        for (int i = 0; i< callItemList.size(); i++) {
            CallItem callItem = callItemList.get(i);
            String component = StringUtil.trimHandle(callItem.getComponentBo());
            String callItemNo = callItem.getSfc() + callItem.getStepId() + callItem.getOperation() + component;
            callItem.setCallItemNo(callItemNo);
            callItem.setHandle(HandleEnum.CALL_ITEM.getHandle(site, callItemNo));
            callItem.setCallType(Constants.CALL_TYPE_MATERIAL);
            callItem.setStatus(Constants.STATUS_NEW);
            callItem.setCallStatus(Constants.STATUS_NEW);
            callItem.setIssueStatus(Constants.STATUS_NEW);
            callItem.setCreateUser(user);
            callItem.setCreatedDateTime(nowDate);
            callItem.setUpdateUser(user);
            callItem.setUpdatedDateTime(nowDate);
            list.add(callItem);
        }


        //数据保存
        if (list != null && list.size() > 0) {
            this.saveBatch(list);
        }
    }

    @Override
    public List<CallItem> findCallItemList(CallItem callItem) {
        return callItemMapper.findCallItemList(callItem);
    }

    @Override
    public List<CallItem> findCallItemDetailList(CallItem callItem) {
        return callItemMapper.findCallItemDetailList(callItem);
    }

    @Override
    public List<CallItem> findCallItemDetailListByDispatchBo(String site, List<String> dispatchBoList) {
        return callItemMapper.findCallItemDetailListByDispatchBo(site, dispatchBoList);
    }

    @Override
    public void callItem(String site, String user, List<CallItem> callItemList) {
        if (callItemList == null || callItemList.size() <= 0) {
            return;
        }

        LocalDateTime dateTime = LocalDateTime.now();
        callItemMapper.updateCallItemStatusByDispatchBo(Constants.CALL_ITEM_STATUS_CALLED, user, dateTime, callItemList);
    }

    @Override
    public void confirmReceive(String site, String user, List<CallItem> callItemList) {
        if (callItemList == null || callItemList.size() <= 0) {
            return;
        }

        for (CallItem callItem : callItemList) {
            String sfcDispatchBo = callItem.getSfcDispatchBo();
            List<CallItem> list = callItemMapper.selectCallItemAndType(sfcDispatchBo);
            if (list == null || list.size() <= 0) {
                continue;
            }

            for (CallItem callItem1 : list) {
                String callType = callItem1.getCallType();
                String materialType = callItem1.getMaterialType();
                if (Constants.CALL_TYPE_MATERIAL.equals(callType) && "ROH".equals(materialType)) {
                    throw BusinessException.build("勾选的派工单包含原材料组件，不能触发确认到料");
                }
            }
        }

        LocalDateTime dateTime = LocalDateTime.now();
        callItemMapper.updateCallItemStatusByDispatchBo(Constants.CALL_ITEM_STATUS_ISSUED, user, dateTime, callItemList);
    }

    @Override
    public void cancelCallItem(String site, String user, List<String> callItemNoList) {
        if (callItemNoList == null || callItemNoList.size() <= 0) {
            return;
        }

        LocalDateTime dateTime = LocalDateTime.now();
        callItemMapper.updateCallItemStatusByCallItemNo(Constants.CALL_ITEM_STATUS_CANCELED, user, dateTime, callItemNoList);
    }

    @Override
    public void refreshRequiredTime(String site, String user, List<String> callItemNoList) {
        if (callItemNoList == null || callItemNoList.size() <= 0) {
            return;
        }
        callItemNoList.stream().distinct().collect(Collectors.toList());

        LocalDateTime dateTime = LocalDateTime.now();
        callItemMapper.refreshRequiredTime(user, dateTime, callItemNoList);
    }


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    @Override
    public List<CallItem> findIssueItemList(IssueItemDto issueItemDto) {
        if (issueItemDto.getMatchResource()) {
            return callItemMapper.finsIssueItemListToResource(issueItemDto);
        } else {
            return callItemMapper.finsIssueItemListNoResource(issueItemDto);
        }
    }
}