package com.foreverwin.mesnac.dispatch.service.impl;

import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.util.ExcelUtils;
import com.foreverwin.mesnac.common.util.ExceptionUtil;
import com.foreverwin.mesnac.common.util.NumberUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.model.Inventory;
import com.foreverwin.mesnac.meapi.service.InventoryService;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.meext.MEServices;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.dispatch.model.SurplusItemReturn;
import com.foreverwin.mesnac.dispatch.mapper.SurplusItemReturnMapper;
import com.foreverwin.mesnac.dispatch.service.SurplusItemReturnService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sap.me.inventory.InventoryServiceInterface;
import com.sap.me.inventory.InventorySplitRequest;
import com.sap.me.inventory.NewSplitInventory;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * <p>
 * 余料退回 服务实现类
 * </p>
 *
 * @author Leon.L
 * @since 2021-07-10
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SurplusItemReturnServiceImpl extends ServiceImpl<SurplusItemReturnMapper, SurplusItemReturn> implements SurplusItemReturnService {


    @Autowired
    private InventoryService inventoryService;
    @Autowired
    private SurplusItemReturnMapper surplusItemReturnMapper;


    @Override
    public Map<String, Object> getInventoryData(String site, String inventory) {
        String inventoryBo = HandleEnum.INVENTORY.getHandle(site, inventory);
        Inventory inventoryModel = inventoryService.getById(inventoryBo);
        if (inventoryModel == null) {
            throw BusinessException.build("物料条码【" + inventory +"】不存在！");
        }
        if (inventoryModel.getQtyOnHand().compareTo(BigDecimal.ZERO) <= 0) {
            throw BusinessException.build("物料条码【" + inventory +"】没有可用数量！");
        }

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Map<String, Object> map = surplusItemReturnMapper.selectInventoryData(inventoryBo);
        if (map == null || map.size() <= 0) {
            throw BusinessException.build("物料条码【" + inventory +"】不存在！");
        }
        String item = (String) map.get("ITEM");
        String perMater = (String) map.get("PER_METER");
        String perSquareMeter = (String) map.get("PER_SQUARE_METER");
        if (StringUtil.isBlank(perMater) && StringUtil.isBlank(perSquareMeter)) {
            throw BusinessException.build("物料【" +item+ "】的自定义字段【每米重量】和【每平米重量】都没维护！");
        }
        if (StringUtil.notBlank(perMater) && !NumberUtil.isNumber(perMater)) {
            throw BusinessException.build("物料【" +item+ "】的自定义字段【每米重量】只能维护数值！");
        }
        if (StringUtil.notBlank(perSquareMeter) && !NumberUtil.isNumber(perSquareMeter)) {
            throw BusinessException.build("物料【" +item+ "】的自定义字段【每平米重量】只能维护数值！");
        }

        return map;
    }

    @Override
    public void surplusReturn(String site, String user, String inventory, String length, String width, BigDecimal qty) {
        String inventoryBo = HandleEnum.INVENTORY.getHandle(site, inventory);
        Inventory inventoryModel = inventoryService.getById(inventoryBo);
        if (inventoryModel == null) {
            throw BusinessException.build("物料条码【" + inventory +"】不存在！");
        }
        BigDecimal qtyOnHand = inventoryModel.getQtyOnHand();
        if (qtyOnHand.compareTo(BigDecimal.ZERO) <= 0) {
            throw BusinessException.build("物料条码【" + inventory +"】没有可用数量！");
        }
        if (qtyOnHand.compareTo(qty) < 0) {
            throw BusinessException.build("余料数量超过了物料条码现有数量【" +qtyOnHand+ "】！");
        }

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        InventoryServiceInterface meInventoryService;
        try {
            meInventoryService = MEServices.create("com.sap.me.inventory", "InventoryService", site);
        } catch (Exception e) {
            throw BusinessException.build("获取标准库存操作服务失败：" + e.getMessage());
        }


        //余料库存
        StringBuffer _inventory = new StringBuffer(inventory);
        if (StringUtil.notBlank(length)) {
            _inventory.append("*"+length);
        }
        if (StringUtil.notBlank(width)) {
            _inventory.append("*"+width);
        }
        String surplusInventory = _inventory.toString();

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        //库存拆分
        try {
            InventorySplitRequest request = new InventorySplitRequest();
            request.setInventoryRef(inventoryBo);
            request.setQuantityToSplit(qty);

            List<NewSplitInventory> splitList = new ArrayList<>();
            NewSplitInventory newSplitInventory = new NewSplitInventory();
            newSplitInventory.setNewInventoryId(surplusInventory);
            newSplitInventory.setQuantity(qty);
            splitList.add(newSplitInventory);
            request.setNewInventoryList(splitList);
            meInventoryService.split(request);
        } catch (Exception e) {
            ExceptionUtil.throwException(e);
        }

        LocalDateTime dateTime = LocalDateTime.now();
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        //记录余料退回日志
        SurplusItemReturn surplusItemReturn = new SurplusItemReturn();
        surplusItemReturn.setHandle(UUID.randomUUID().toString());
        surplusItemReturn.setSite(site);
        surplusItemReturn.setParentInventory(inventory);
        surplusItemReturn.setInventory(surplusInventory);
        surplusItemReturn.setLenght(length);
        surplusItemReturn.setWidth(width);
        surplusItemReturn.setItemBo(inventoryModel.getItemBo());
        surplusItemReturn.setQty(qty);
        surplusItemReturn.setCreateUser(user);
        surplusItemReturn.setCreatedDateTime(dateTime);
        surplusItemReturn.setModifuUser(user);
        surplusItemReturn.setModifiedDateTime(dateTime);
        surplusItemReturnMapper.insert(surplusItemReturn);
    }
}