package com.foreverwin.mesnac.equip.consumer;

import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.equip.dto.MQResponseDto;
import com.foreverwin.mesnac.meapi.service.ResrceService;
import com.foreverwin.modular.core.exception.BusinessException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 设备接口业务处理
 *
 *
 * @author Leon
 * @date 2021-07-21
 */
@Component
public class ResourceMQProcess {


    @Autowired
    private ResrceService resrceService;

    /**
     * 设备状态变更
     *
     *
     * 设备状态：运行/RUN(301|已启用)；停机/DOW(302|已禁用);维修/REPAIR(4/预定停机);封存/SEAL(303|保留);空闲/FREE(2|备用)
     * @param text
     * @return
     */
    public String resourceStatusProcess(String text) {
        MQResponseDto mqResponseDto = new MQResponseDto();

        try {
            JSONObject jsonObject = JSONObject.parseObject(text);
            String handle = jsonObject.getString("TRANID");
            mqResponseDto.setHANDLE(handle);

            String site = jsonObject.getString("SITE");
            String resource = jsonObject.getString("RESOURCE");
            String status = jsonObject.getString("STATUS");
            if (StringUtil.isBlank(site)) {
                throw BusinessException.build("站点不能为空！");
            }
            if (StringUtil.isBlank(resource)) {
                throw BusinessException.build("设备编号不能为空！");
            }
            if (StringUtil.isBlank(status)) {
                throw BusinessException.build("设备状态不能为空！");
            }

            String meStatus = "301";
            //设备状态转换
            switch (status) {
                case "RUN" :
                    meStatus = "301"; break;
                case "DOW" :
                    meStatus = "302"; break;
                case "SEAL" :
                    meStatus = "303"; break;
                case "FREE" :
                    meStatus = "2"; break;
                case "REPAIR" :
                    meStatus = "4"; break;
            }

            resrceService.updateResourceStatus(site, resource, meStatus);
            mqResponseDto.setSTATUS("S");
            mqResponseDto.setMESSAGE("设备状态修改成功！");
        } catch (Exception e) {
            mqResponseDto.setSTATUS("E");
            mqResponseDto.setMESSAGE("设备状态修改失败：" + e.getMessage());
        }

        return JSONObject.toJSONString(mqResponseDto);
    }
}
