package com.foreverwin.mesnac.production.service.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.enums.DispatchStatusEnum;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.model.InspectionItemDetail;
import com.foreverwin.mesnac.common.model.InspectionTask;
import com.foreverwin.mesnac.common.service.CommonService;
import com.foreverwin.mesnac.common.service.InspectionItemService;
import com.foreverwin.mesnac.common.service.InspectionTaskService;
import com.foreverwin.mesnac.common.service.SfcDispatchCommonService;
import com.foreverwin.mesnac.common.util.ExceptionUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.dto.SfcDto;
import com.foreverwin.mesnac.meapi.dto.WorkCenterDto;
import com.foreverwin.mesnac.meapi.model.Operation;
import com.foreverwin.mesnac.meapi.model.Resrce;
import com.foreverwin.mesnac.meapi.model.Sfc;
import com.foreverwin.mesnac.meapi.service.OperationService;
import com.foreverwin.mesnac.meapi.service.ResrceService;
import com.foreverwin.mesnac.meapi.service.SfcService;
import com.foreverwin.mesnac.production.mapper.SfcCrossMapper;
import com.foreverwin.mesnac.production.service.PodTemplateService;
import com.foreverwin.mesnac.production.service.SfcCrossService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.CommonMethods;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Service
@Transactional(rollbackFor = Exception.class)
public class PodTemplateServiceImpl implements PodTemplateService {
    @Autowired
    private ResrceService resrceService;
    @Autowired
    private SfcService sfcService;
    @Autowired
    private CommonService commonService;
    @Autowired
    private SfcCrossMapper sfcCrossMapper;
    @Autowired
    private SfcCrossService sfcCrossService;
    @Autowired
    private OperationService operationService;
    @Autowired
    private InspectionTaskService inspectionTaskService;
    @Autowired
    private InspectionItemService inspectionItemService;
    @Autowired
    private SfcDispatchCommonService sfcDispatchCommonService;
    @Override
    public Map<String, Object> resrceEnter(WorkCenterDto workCenterDto) {
        String site = CommonMethods.getSite();
        String resrce = workCenterDto.getResrce();
        String resrceBO = HandleEnum.RESOURCE.getHandle(site, resrce);
        Resrce model = resrceService.getById(resrceBO);
        if (model == null) {
            throw new BaseException("资源 " + resrce + " 不存在");
        }
        String workCenter = workCenterDto.getWorkCenter();
        //校验
        String workShopBo = commonService.getWorkShopBo(resrceBO);
        if (StringUtil.isBlank(workShopBo)) {
            throw new BaseException("资源 " + resrce + " 未匹配到车间");
        }
        if (!workCenter.equals(StringUtil.trimHandle(workShopBo))) {
            throw new BaseException("资源 " + resrce + " 与车间不匹配");
        }
        //查询在该设备存在活动中的SFC
        List<SfcDto> sfcList = sfcCrossService.getSfcListByResrce(site,resrce);
        Map<String, Object> resultMap = new HashMap<String, Object>();
        resultMap.put("SFC_LIST", sfcList);
        return resultMap;
    }

    @Override
    public Map<String, Object> sfcEnter(SfcDto sfcDto) {
        String site = CommonMethods.getSite();
        String sfc = sfcDto.getSfc();
        String locale = LocaleContextHolder.getLocale().getLanguage();
        String resrce = sfcDto.getResrce();
        //校验产品条码是否存在
        Sfc sfcById = sfcService.getById(HandleEnum.SFC.getHandle(site, sfc));
        if (sfcById == null) {
            throw new BaseException("产品条码" + sfc + "不存在");
        }
        String status = StringUtil.trimHandle(sfcById.getStatusBo());
        if (!status.equals("403") && !status.equals("401") && !status.equals("402")) {
            throw new BaseException("产品条码" + sfc + "不在工作中或排队中");
        }
        //校验产品条码是否在当前工序排队
        Operation operationBySfcBo = commonService.getOperationBySfcBo(sfcById.getHandle());
       /* if (StringUtil.notBlank(operation)&&!operationBySfcBo.getOperation().equals(operation)){
            throw new BaseException("产品条码"+sfc+"在"+operationBySfcBo.getOperation()+"/"+operationBySfcBo.getDescription()+"工序");
        }*/
        //校验SFC+工序+设备+STEP_ID是否与当前设备确认的派工单匹配
        Integer integer = sfcCrossMapper.getSfcDispatch(site, sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId(), resrce);
        if (integer < 1) {
            throw new BaseException("产品条码与当前设备工序确认的派工单不匹配");
        }

        //构造前台所需要的数据
        Map<String, Object> resultMap = sfcCrossMapper.querySfcData(site, locale, sfcDto);
        if (resultMap == null) {
            throw new BaseException("根据当前资源未找到条码[" + sfc + "]的基本信息！");
        }
        //List<Map<String, Object>> substepList = sfcCrossMapper.querySfcStep(site, sfc, operationBySfcBo.getOperation());
        List<Map<String, Object>> substepList = new ArrayList<>();
        resultMap.put("SFC_STEP_LIST", substepList);

        //是否需要自检和互检
        String isCreateH = "N";
        String isCreateZ = "N";
        resultMap.put("IS_CREATE_W", "N");
        List<InspectionItemDetail> inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId(), Constants.INSPECTION_TYPE_H);
        if (inspectionItemDetails.size()>0&&inspectionItemDetails.get(0)!=null) {
            InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_H, sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId());
            if (createTask==null ||!createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)){
                isCreateH="Y";
            }

        }
        inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId(), Constants.INSPECTION_TYPE_Z);
        if (inspectionItemDetails.size()>0&& inspectionItemDetails.get(0)!=null) {
            InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_Z, sfc, operationBySfcBo.getOperation(), operationBySfcBo.getStepId());
            if (createTask==null ||!createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)){
                isCreateZ = "Y";
            }
        }
        resultMap.put("IS_CREATE_H", isCreateH);
        resultMap.put("IS_CREATE_Z", isCreateZ);
        return resultMap;
    }

    @Override
    public Object getSfcInfo(SfcDto sfcDto) {
        String site = CommonMethods.getSite();
        String operation = sfcDto.getOperation();
        String sfc = sfcDto.getSfc();
        Map<String, Object> substepList = sfcCrossMapper.getSfcInfo(site, sfc, operation);
        return substepList;
    }

    @Override
    public void sfcStart(Map<String, Object> map) {
        List<SfcDto> sfcDtoList = (List<SfcDto>) map.get("sfcDtoList");
        ObjectMapper mapper = new ObjectMapper();
        sfcDtoList = mapper.convertValue(sfcDtoList, new TypeReference<List<SfcDto>>() {});
        String resrce = (String) map.get("resrce");
        String site = CommonMethods.getSite();

        if (sfcDtoList==null||sfcDtoList.size()<1){
            throw new BaseException("作业列表不能为空");
        }
        if (StringUtil.isBlank(resrce)){
            throw new BaseException("资源不能为空");
        }
        //是否设备点检
        sfcCrossService.resourceCheck(HandleEnum.RESOURCE.getHandle(site,resrce));
        sfcDtoList.forEach(sfcDto -> {
            String operation = sfcDto.getOperation();
            Operation currentRevisionRef = operationService.getCurrentRevisionRef(site, operation);
            String sfc = sfcDto.getSfc();
            String stepId = sfcDto.getStepId();
            String dispatchNo = sfcDto.getDispatchNo();
            Sfc sfcServiceById = sfcService.getById(HandleEnum.SFC.getHandle(site, sfc));
            BigDecimal qty=new BigDecimal(sfcServiceById.getQty().toString());
            //是否有互检检验项目
            List<InspectionItemDetail> inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operation, stepId, Constants.INSPECTION_TYPE_H);
            if (inspectionItemDetails.size()>0&&inspectionItemDetails.get(0)!=null) {
                //是否有互检检验任务及已完成
                InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_H, sfc, operation, stepId);
                if (createTask==null ||!createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)){
                    throw new BaseException("请完成互检检验任务");
                }
                if (!createTask.getResult().equals(Constants.RSESULT_OK)){
                    throw new BaseException("互检任务不合格，不能开始请检查");
                }
            }
            try {
                sfcCrossService.startAction(site, currentRevisionRef.getHandle(),resrce,sfcServiceById.getHandle(),qty);
            } catch (Exception e) {
                ExceptionUtil.throwException(e);
            }
            //更改派工单状态
            sfcDispatchCommonService.updateSfcDispatchStatus(site,CommonMethods.getUser(),dispatchNo, DispatchStatusEnum.START.getCode());

        });
    }
    @Override
    public void sfcComplete(Map<String, Object> map) {
        List<SfcDto> sfcDtoList = (List<SfcDto>) map.get("sfcDtoList");
        ObjectMapper mapper = new ObjectMapper();
        sfcDtoList = mapper.convertValue(sfcDtoList, new TypeReference<List<SfcDto>>() {});
        String resrce = (String) map.get("resrce");
        if (sfcDtoList==null||sfcDtoList.size()<1){
            throw new BaseException("作业列表不能为空");
        }
        if (StringUtil.isBlank(resrce)){
            throw new BaseException("资源不能为空");
        }

        sfcDtoList.forEach(sfcDto -> {
            String site = CommonMethods.getSite();
            String operation = sfcDto.getOperation();
            Operation currentRevisionRef = operationService.getCurrentRevisionRef(site, operation);
            String sfc = sfcDto.getSfc();
            String stepId = sfcDto.getStepId();

            String dispatchNo = sfcDto.getDispatchNo();
            Sfc sfcServiceById = sfcService.getById(HandleEnum.SFC.getHandle(site, sfc));
            BigDecimal qty=new BigDecimal(sfcServiceById.getQty().toString());
            //是否有自检检验项目
            List<InspectionItemDetail> inspectionItemDetails = inspectionItemService.selectQualityInspection(sfc, operation, stepId, Constants.INSPECTION_TYPE_Z);
            if (inspectionItemDetails.size()>0&&inspectionItemDetails.get(0)!=null) {
                //是否有自检检验任务
                InspectionTask createTask = inspectionTaskService.isCreateTask(site, Constants.INSPECTION_TYPE_Z, sfc, operation, stepId);
                if (createTask==null ||!createTask.getStatus().equals(Constants.INSPECTION_TASK_STATUS_COMPLETE)){
                    throw new BaseException("请完成自检检验任务");
                }
                if (!createTask.getResult().equals(Constants.RSESULT_OK)){
                    throw new BaseException("自检任务不合格，不能完成请检查");
                }
            }
            try {
                sfcCrossService.completeAction(site, currentRevisionRef.getHandle(),resrce,sfcServiceById.getHandle(),qty);
            } catch (Exception e) {
                ExceptionUtil.throwException(e);
            }
            //更改派工单状态
            sfcDispatchCommonService.updateSfcDispatchStatus(site,CommonMethods.getUser(),dispatchNo, DispatchStatusEnum.COMPLETE.getCode());
        });
    }
}
